from la_nlp import components, config
from spacy import load as load_model
from spacy.language import Language

ASPECTS = config.get_aspects()

NLP = load_model('en_core_web_lg')

def make_doc(text, nlp=NLP, aspects=ASPECTS):
    keywords = []
    for keywords_list in aspects.values():
        keywords.extend(keywords_list)
    return nlp(text, component_cfg={'aspect_sentiment_pipe':{'base_aspects':aspects, 'base_keywords':keywords}})

@Language.component('aspect_sentiment_pipe')
def aspect_sentiment_pipe(doc, base_aspects, base_keywords):
    doc = components.contains_aspect(doc, base_keywords)
    doc = components.aspects(doc, base_aspects)
    doc = components.keywords(doc, base_keywords)
    doc = components.keyword_aspects(doc, base_aspects)
    doc = components.parent_span(doc)
    doc = components.parent_span_sentiment(doc)
    doc = components.aspect_sentiments(doc, base_aspects)
    return doc

NLP.add_pipe('aspect_sentiment_pipe')


from la_nlp import config
from spacy.tokens import Doc, Span, Token
from vaderSentiment.vaderSentiment import SentimentIntensityAnalyzer

ANALYZER = SentimentIntensityAnalyzer()

def set_extension(extension_name, default=None, target_obj=Doc):
    if not target_obj.has_extension(extension_name):
        target_obj.set_extension(extension_name, default=default)

def contains_aspect(doc, base_keywords):
    set_extension('contains_aspect', default=False)
    
    for token in doc:
        if token.lemma_.lower() in base_keywords:
            doc._.contains_aspect = True
            break
    
    return doc

def aspects(doc, base_aspects):
    set_extension('aspects')
    
    if doc._.contains_aspect == True:
        aspects_contained = []
        for token in doc:
            for aspect in base_aspects:
                if aspect in aspects_contained:
                    continue
                keywords = base_aspects[aspect]
                if token.lemma_.lower() in keywords:
                    aspects_contained.append(aspect)
        doc._.aspects = aspects_contained
    return doc

def keywords(doc, base_keywords):
    set_extension('keywords')

    if doc._.contains_aspect == True:
        keywords = []
        for token in doc:
            if token.lemma_.lower() in base_keywords:
                keywords.append(token)
        doc._.keywords = keywords
    return doc

def keyword_aspects(doc, base_aspects):
    set_extension('aspect', target_obj=Token)

    if doc._.keywords == None:
        return doc

    for doc_keyword in doc._.keywords:
        for aspect in base_aspects:
            for keyword in base_aspects[aspect]:
                if doc_keyword.lemma_.lower() == keyword:
                    doc_keyword._.aspect = aspect
                continue
            if doc_keyword._.aspect != None:
                continue
        if doc_keyword._.aspect != None:
            continue

    return doc

def parent_span(doc):
    set_extension('parent_span', target_obj=Token)

    for token in doc:
        head = token.head
        siblings = head.children
        indices = [token.i for token in siblings]
        indices.append(head.i)
        indices.sort()
        first = indices[0]
        last = indices[-1] + 1
        if last >= len(doc):
            span = doc[first:]
        else:
            span = doc[first:last]
        token._.parent_span = span          

    return doc

def parent_span_sentiment(doc, include_non_keywords=False):
    set_extension('sentiment', target_obj=Span)

    if include_non_keywords == True:
        tokens = doc
    else:
        tokens = doc._.keywords
    
    if tokens == None:
        return doc
    
    for token in tokens:
        scores = ANALYZER.polarity_scores(token._.parent_span.text)
        sentiment = scores['compound']
        token._.parent_span._.sentiment = sentiment
    
    return doc

def aspect_sentiments(doc, base_aspects):
    set_extension('aspect_sentiments')

    aspect_sentiments = {aspect:None for aspect in base_aspects}

    if doc._.keywords != None:
        for keyword in doc._.keywords:
            aspect = keyword._.aspect
            sentiment = keyword._.parent_span._.sentiment
            aspect_sentiments[aspect] = sentiment
    
    doc._.aspect_sentiments = aspect_sentiments

    return doc



import os
from la_nlp.aspect_sentiment_pipe import *
import pytest
from spacy.tokens import Doc

FILE_DIR = os.path.dirname(__file__)

ASPECTS_1_PATH = os.path.join(FILE_DIR, 'test_data', 'test_aspects_1.toml')
ASPECTS_1 = config.get_aspects(ASPECTS_1_PATH)

ASPECTS_2_PATH = os.path.join(FILE_DIR, 'test_data', 'test_aspects_2.toml')
ASPECTS_2 = config.get_aspects(ASPECTS_2_PATH)

TEST_TEXT_1 = """
I enjoyed the course, but the readings were too long and the professor was mean.
"""

TEST_TEXT_2 = """
This is a text that does not contain any target aspects.
"""

@pytest.fixture
def doc1():
    doc1 = make_doc(TEST_TEXT_1, aspects=ASPECTS_1)
    return doc1

@pytest.fixture
def doc2():
    doc2 = make_doc(TEST_TEXT_2, aspects=ASPECTS_1)
    return doc2

def test_make_doc(doc1):
    assertion1 = "Should be a spacy Doc object"
    assert isinstance(doc1, Doc), assertion1

    assertion2 = "Should have a length of 19"
    assert len(doc1) == 19, assertion2


def test_contains_aspect(doc1, doc2):
    assertion1 = "doc1 should return True"
    assert doc1._.contains_aspect == True, assertion1

    assertion2 = "doc2 should return False"
    assert doc2._.contains_aspect == False, assertion2

def test_aspects(doc1, doc2):
    assertion1 = "doc1 should contain 'course', 'content', and 'instructor'"
    doc1_aspects = ['course', 'content', 'instructor']
    assert doc1._.aspects == doc1_aspects, assertion1

    assertion2 = "doc2 should contain None"
    doc2_aspects = None
    assert doc2._.aspects == doc2_aspects, assertion2

def test_keywords(doc1, doc2):
    assertion1 = "doc1 should contain keyword %s"
    doc1_targets = ['course', 'reading', 'professor']
    doc1_keywords = [kw.lemma_.lower() for kw in doc1._.keywords]
    for target in doc1_targets:
        assert target in doc1_keywords, assertion1 % target
    
    assertion2 = "doc2 should contain no keywords"
    doc2_targets = None
    assert doc2._.keywords == doc2_targets, assertion2

def test_keyword_aspects(doc1):
    assertion = "%s aspect should be %s"
    target_aspects = ['course', 'content', 'instructor']
    for keyword, aspect in zip(doc1._.keywords, target_aspects):
        assert keyword._.aspect == aspect, assertion % (keyword.text, aspect)


def test_parent_span(doc1, doc2):
    assertion1 = "Span should read 'professor was mean'"
    doc1_target = 'professor was mean'
    token1 = doc1._.keywords[2]
    assert token1._.parent_span.text == doc1_target, assertion1

    assertion2 = "Span should read 'that does not contain any target aspects'"
    doc2_target = 'that does not contain any target aspects'
    token2 = doc2[-3]
    assert token2._.parent_span.text == doc2_target, assertion2

def test_parent_span_sentiment(doc1):
    assertion = "course parent span sentiment should be 0.2846"
    span = doc1._.keywords[0]._.parent_span
    target_sentiment = 0.2846

    assert span._.sentiment == target_sentiment, assertion

def test_aspect_sentiments(doc1):
    target_sentiments = {
        'course': 0.2846,
        'content': 0,
        'assignments':None,
        'tests':None,
        'instructor': 0,
    }
    assertion = f"Sentiment dict should look like: {target_sentiments}"

    assert doc1._.aspect_sentiments == target_sentiments, assertion



import tomllib
import os

FILE_DIR = os.path.dirname(__file__)
PROJECT_DIR = os.path.dirname(FILE_DIR)
DATA_DIR = os.path.join(PROJECT_DIR, 'data')
ASPECT_FILE = os.path.join(DATA_DIR, 'aspects.toml')

def get_aspects(file_path=ASPECT_FILE):
    with open(file_path, 'rb') as file:
        aspects = tomllib.load(file)
    return aspects
